@@ -26,7 +26,7 @@
 
 extern const char *minor_str[];
 
-#define VERSION "v5.1"
+#define VERSION "v5.2"
 
 #undef DO_FLUSHCACHE		/* under construction: force cache flush on -W0 */
 
@@ -52,7 +52,7 @@ extern const char *minor_str[];
 char *progname;
 static int verbose = 0, get_identity = 0, get_geom = 0, noisy = 1, quiet = 0;
 static int flagcount = 0, do_flush = 0, is_scsi_hd = 0, is_xt_hd = 0;
-static int do_ctimings, do_timings = 0, old_identity = 0, timing_MB = TIMING_MB;
+static int do_ctimings, do_timings = 0, timing_MB = TIMING_MB;
 
 static unsigned long set_readahead= 0, get_readahead= 0, readahead= 0;     
 static unsigned long set_readonly = 0, get_readonly = 0, readonly = 0;
@@ -124,6 +124,17 @@ static int	perform_reset = 0;
 static int	perform_tristate = 0,	tristate = 0;
 #endif /* HDIO_TRISTATE_HWIF */
 
+// Historically, if there was no HDIO_OBSOLETE_IDENTITY, then
+// then the HDIO_GET_IDENTITY only returned 142 bytes.
+// Otherwise, HDIO_OBSOLETE_IDENTITY returns 142 bytes,
+// and HDIO_GET_IDENTITY returns 512 bytes.  But the latest
+// 2.5.xx kernels no longer define HDIO_OBSOLETE_IDENTITY
+// (which they should, but they should just return -EINVAL).
+//
+// So.. we must now assume that HDIO_GET_IDENTITY returns 512 bytes.
+// On a really old system, it will not, and we will be confused.
+// Too bad, really.
+
 const char *cfg_str[] =
 {	"",	     " HardSect",   " SoftSect",  " NotMFM",
 	" HdSw>15uSec", " SpinMotCtl", " Fixed",     " Removeable",
@@ -227,35 +238,29 @@ static void dump_identity (const struct hd_driveid *id)
 			if (id->eide_pio_modes & 2)	strcat(pmodes, "pio4 ");
 			if (id->eide_pio_modes &~3)	strcat(pmodes, "pio? ");
 		}
-#ifdef HDIO_OBSOLETE_IDENTITY
 		if (id->field_valid&4) {
-			if (old_identity) {
-				strcat(dmodes," udma??");
-			} else {
-				if (id->dma_ultra & 0x100)	strcat(umodes,"*");
-				if (id->dma_ultra & 0x001)	strcat(umodes,"udma0 ");
-				if (id->dma_ultra & 0x200)	strcat(umodes,"*");
-				if (id->dma_ultra & 0x002)	strcat(umodes,"udma1 ");
-				if (id->dma_ultra & 0x400)	strcat(umodes,"*");
-				if (id->dma_ultra & 0x004)	strcat(umodes,"udma2 ");
+			if (id->dma_ultra & 0x100)	strcat(umodes,"*");
+			if (id->dma_ultra & 0x001)	strcat(umodes,"udma0 ");
+			if (id->dma_ultra & 0x200)	strcat(umodes,"*");
+			if (id->dma_ultra & 0x002)	strcat(umodes,"udma1 ");
+			if (id->dma_ultra & 0x400)	strcat(umodes,"*");
+			if (id->dma_ultra & 0x004)	strcat(umodes,"udma2 ");
 #ifdef __NEW_HD_DRIVE_ID
-				if (id->hw_config & 0x2000) {
+			if (id->hw_config & 0x2000) {
 #else /* !__NEW_HD_DRIVE_ID */
-				if (id->word93 & 0x2000) {
+			if (id->word93 & 0x2000) {
 #endif /* __NEW_HD_DRIVE_ID */
-					if (id->dma_ultra & 0x800)	strcat(umodes,"*");
-					if (id->dma_ultra & 0x008)	strcat(umodes,"udma3 ");
-					if (id->dma_ultra & 0x1000)	strcat(umodes,"*");
-					if (id->dma_ultra & 0x0010)	strcat(umodes,"udma4 ");
-					if (id->dma_ultra & 0x2000)	strcat(umodes,"*");
-					if (id->dma_ultra & 0x0020)	strcat(umodes,"udma5 ");
-					if (id->dma_ultra & 0x4000)	strcat(umodes,"*");
-					if (id->dma_ultra & 0x0040)	strcat(umodes,"udma6 ");
-					if (id->dma_ultra & 0x8000)	strcat(umodes,"*");
-					if (id->dma_ultra & 0x0080)	strcat(umodes,"udma7 ");
-				}
+				if (id->dma_ultra & 0x0800)	strcat(umodes,"*");
+				if (id->dma_ultra & 0x0008)	strcat(umodes,"udma3 ");
+				if (id->dma_ultra & 0x1000)	strcat(umodes,"*");
+				if (id->dma_ultra & 0x0010)	strcat(umodes,"udma4 ");
+				if (id->dma_ultra & 0x2000)	strcat(umodes,"*");
+				if (id->dma_ultra & 0x0020)	strcat(umodes,"udma5 ");
+				if (id->dma_ultra & 0x4000)	strcat(umodes,"*");
+				if (id->dma_ultra & 0x0040)	strcat(umodes,"udma6 ");
+				if (id->dma_ultra & 0x8000)	strcat(umodes,"*");
+				if (id->dma_ultra & 0x0080)	strcat(umodes,"udma7 ");
 			}
-#endif
 		}
 	}
 	if ((id->capability&1) && (id->field_valid&2))
@@ -1197,13 +1202,13 @@ void process_dev (char *devname)
 	if (verbose || get_geom) {
 		static const char msg[] = " geometry     = %u/%u/%u, sectors = %ld, start = %ld\n";
 		static struct hd_geometry g;
-#if HDIO_GETGEO_BIG
+#ifdef HDIO_GETGEO_BIG
 		static struct hd_big_geometry bg;
 #endif
 
 		if (ioctl(fd, BLKGETSIZE, &parm))
 			perror(" BLKGETSIZE failed");
-#if HDIO_GETGEO_BIG
+#ifdef HDIO_GETGEO_BIG
 		else if (!ioctl(fd, HDIO_GETGEO_BIG, &bg))
 			printf(msg, bg.cylinders, bg.heads, bg.sectors, parm, bg.start);
 #endif
@@ -1257,12 +1262,8 @@ void process_dev (char *devname)
 
 		no_scsi();
 		no_xt();
-#ifdef HDIO_OBSOLETE_IDENTITY
-		if (!(old_identity = ioctl(fd, HDIO_GET_IDENTITY, &id)) || !ioctl(fd, HDIO_OBSOLETE_IDENTITY)) {
-#else
+
 		if (!ioctl(fd, HDIO_GET_IDENTITY, &id)) {
-			old_identity = 1;
-#endif
 			if (multcount != -1) {
 				id.multsect = multcount;
 				id.multsect_valid |= 1;
@@ -1785,8 +1786,8 @@ int main(int argc, char **argv)
 					case 'Q':
 						get_dma_q = noisy;
 						noisy = 1;
-#ifdef HDIO_SET_QDMA
 						neg = 0;
+#ifdef HDIO_SET_QDMA
 						GET_NUMBER(set_dma_q, dma_q);
 						if (neg)
 							dma_q = -dma_q;
